```@meta
CurrentModule = Antique
```

# Harmonic Oscillator

The harmonic oscillator is the most frequently used model in quantum physics.

## Definitions

This model is described with the time-independent Schrödinger equation
```math
  \hat{H} \psi(x) = E \psi(x),
```
and the Hamiltonian
```math
  \hat{H} = - \frac{\hbar^2}{2m} \frac{\mathrm{d}^2}{\mathrm{d}x ^2} + V(x).
```
Parameters are specified with the following struct.

#### Parameters
```@docs; canonical=false
Antique.HarmonicOscillator
```

#### Potential
```@docs; canonical=false
Antique.V(::HarmonicOscillator, ::Any)
```

#### Eigen Values
```@docs; canonical=false
Antique.E(::HarmonicOscillator)
```

#### Eigen Functions
```@docs; canonical=false
Antique.ψ(::HarmonicOscillator, ::Any)
```

#### Hermite Polynomials
```@docs; canonical=false
Antique.H(::HarmonicOscillator, ::Any)
```

#### Reference
- [DLMF 18.5.18](https://dlmf.nist.gov/18.5#E18)
- [cpprefjp](https://cpprefjp.github.io/reference/cmath/hermite.html)
- The Digital Library of Mathematical Functions (DLMF)                                                    [18.3 Table1](https://dlmf.nist.gov/18.3#T1), [18.5 Table1](https://dlmf.nist.gov/18.5#T1), [18.5.13](https://dlmf.nist.gov/18.5#E13), [18.5.18](https://dlmf.nist.gov/18.5#E18)
- L. D. Landau, E. M. Lifshitz, Quantum Mechanics (Pergamon Press, 1965)                                  [p.595 (a.4), (a.6)](https://archive.org/details/ost-physics-landaulifshitz-quantummechanics/page/n607/mode/2up)
- L. I. Schiff, Quantum Mechanics (McGraw-Hill Book Company, 1968)                                        [p.71 (13.12)](https://archive.org/details/ost-physics-schiff-quantummechanics/page/n87/mode/2up)
- A. Messiah, Quanfum Mechanics (Dover Publications, 1999)                                                [p.491 (B.59)](https://archive.org/details/quantummechanics0000mess/page/491/mode/1up)
- W. Greiner, Quantum Mechanics: An Introduction Third Edition (Springer, 1994)                           [p.152 (7.22)](https://archive.org/details/quantummechanics0001grei_u4x0/page/152/mode/1up)
- D. J. Griffiths, Introduction to Quantum Mechanics (Prentice Hall, 1995)                                [p.41 Table 2.1](https://archive.org/details/griffiths-introduction-to-quantum-mechanics/page/41/mode/1up), [p.43 (2.70)](https://archive.org/details/griffiths-introduction-to-quantum-mechanics/page/43/mode/1up)
- D. A. McQuarrie, J. D. Simon, Physical Chemistry: A Molecular Approach (University Science Books, 1997) [p.170 Table 5.2](https://archive.org/details/McQuarrieSimonPhysicalChemistrySolutions/McQuarrie_Simon_Physical_Chemistry1997/page/n193/mode/1up)
- P. W. Atkins, J. De Paula, Atkins' Physical Chemistry, 8th edition (W. H. Freeman, 2008)                [p.293 Table 9.1](https://archive.org/details/atkinsphysicalch00pwat/page/292/mode/2up)
- J. J. Sakurai, J. Napolitano, Modern Quantum Mechanics Third Edition (Cambridge University Press, 2021) [p.524 (B.29)](https://doi.org/10.1017/9781108587280)

## Usage & Examples

[Install Antique.jl](@ref Install) for the first use and run `using Antique` before each use. The energy `E()`, wavefunction `ψ()`, potential `V()` and some other functions are suppoted. In this system, the model is generated by `HarmonicOscillator` and several parameters `k`, `m` and `ℏ` are set as optional arguments.

```@example HO
using Antique
HO = HarmonicOscillator(k=1.0, m=1.0, ℏ=1.0)
; #hide
```

Parameters:

```@repl HO
HO.k
HO.m
HO.ℏ
```

Eigen values:

```@repl HO
E(HO, n=0)
E(HO, n=1)
```

Potential energy curve:

```@example HO
using CairoMakie

f = Figure()
ax = Axis(f[1,1], xlabel=L"$x$", ylabel=L"$V(x)$")
lines!(ax, -5..5, x -> V(HO, x))
f
```

Wave functions:

```@example HO
using CairoMakie

# setting
f = Figure()
ax = Axis(f[1,1], xlabel=L"$x$", ylabel=L"$\psi(x)$")

# plot
w0 = lines!(ax, -5..5, x -> ψ(HO, x, n=0))
w1 = lines!(ax, -5..5, x -> ψ(HO, x, n=1))
w2 = lines!(ax, -5..5, x -> ψ(HO, x, n=2))
w3 = lines!(ax, -5..5, x -> ψ(HO, x, n=3))
w4 = lines!(ax, -5..5, x -> ψ(HO, x, n=4))

# legend
axislegend(ax, [w0, w1, w2, w3, w4], [L"n=0", L"n=1", L"n=2", L"n=3", L"n=4"], position=:lb)

f
```

Potential energy curve, Energy levels, Wave functions:

```@example HO
using CairoMakie

# settings
f = Figure()
ax = Axis(f[1,1], xlabel=L"$x$", ylabel=L"$V(x),~E_n,~\psi_n(x) \times 5 + E_n$", aspect=1, limits=(-5,5,0,5.2))
# hidespines!(ax)
# hidedecorations!(ax)

for n in 0:4
  # classical turning point
  xE = sqrt(2*HO.k*E(HO, n=n))
  # energy
  lines!(ax, [-xE,xE], fill(E(HO,n=n),2), color=:black, linewidth=2)
  hlines!(ax, E(HO, n=n), color=:black, linewidth=1, linestyle=:dash)
  # wave function
  lines!(ax, -5..5, x -> E(HO,n=n) + 0.5*ψ(HO,x,n=n), linewidth=2)
end

#potential
lines!(ax, -5..5, x -> V(HO, x), color=:black, linewidth=2)

f
save("assets/fig/HarmonicOscillator.png", f) # hide
; # hide
```

![](assets/fig/HarmonicOscillator.png)

## Testing

Unit testing and Integration testing were done using computer algebra system ([Symbolics.jl](https://symbolics.juliasymbolics.org/stable/)) and numerical integration ([QuadGK.jl](https://juliamath.github.io/QuadGK.jl/stable/)). The test script is [here](https://github.com/ohno/Antique.jl/blob/main/test/HarmonicOscillator.jl).

```@eval
using Markdown
using Antique
Markdown.parse(Antique.load("../../test/result/HarmonicOscillator.log"))
```
