```@meta
CurrentModule = Antique
```

# Infinite Potential Well 3D (Particle in a 3D-Box)

The three-dimensional infinite potential well (particle in a 3D-box) is one of the simplest models for a three-dimensional quantum mechanical system.

## Definitions

This model is described with the time-independent Schrödinger equation
```math
  \hat{H} \psi(x,y,z) = E \psi(x,y,z),
```
and the Hamiltonian
```math
  \hat{H} = - \frac{\hbar^2}{2m} \left(\frac{\partial^2}{\partial x ^2} + \frac{\partial^2}{\partial y ^2} + \frac{\partial^2}{\partial z ^2}\right) + V(x,y,z).
```
Parameters are specified with the following struct.

#### Parameters
```@docs; canonical=false
Antique.InfinitePotentialWell3D
```

#### Potential
```@docs; canonical=false
Antique.V(::InfinitePotentialWell3D, ::Any, ::Any, ::Any)
```

#### Eigen Values
```@docs; canonical=false
Antique.E(::InfinitePotentialWell3D)
```

#### Eigen Functions
```@docs; canonical=false
Antique.ψ(::InfinitePotentialWell3D, ::Any, ::Any, ::Any)
```

## Usage & Examples

[Install Antique.jl](@ref Install) for the first use and run `using Antique` before each use. The energy `E()`, wavefunction `ψ()`, potential `V()` and some other functions are supported. In this system, the model is generated by `InfinitePotentialWell3D` and several parameters `Lx`, `Ly`, `Lz`, `m` and `ℏ` are set as optional arguments.

```@example IPW3D
using Antique
IPW3D = InfinitePotentialWell3D(Lx=1.0, Ly=1.0, Lz=1.0, m=1.0, ℏ=1.0)
; #hide
```

Parameters:

```@repl IPW3D
IPW3D.Lx
IPW3D.Ly
IPW3D.Lz
IPW3D.m
IPW3D.ℏ
```

Eigen values:

```@repl IPW3D
E(IPW3D, nx=1, ny=1, nz=1)
E(IPW3D, nx=2, ny=1, nz=1)
E(IPW3D, nx=1, ny=2, nz=1)
E(IPW3D, nx=1, ny=1, nz=2)
E(IPW3D, nx=2, ny=2, nz=1)
E(IPW3D, nx=2, ny=1, nz=2)
E(IPW3D, nx=1, ny=2, nz=2)
E(IPW3D, nx=2, ny=2, nz=2)
```

Wave functions:

The wave functions of the 3D infinite potential well can be expressed as a product of wave functions in 1D infinite potential wells for each of the three directions ``x``,``y``,``z``.
As the representation of 3D (non-spherical) wave functions easily becomes unclear, we refer to the documentation of the one-dimensional counterpart: InfinitePotentialWell (...)



## Testing

Unit testing and Integration testing were done using numerical integration ([QuadGK.jl](https://juliamath.github.io/QuadGK.jl/stable/)). The test script is [here](https://github.com/ohno/Antique.jl/blob/main/test/InfinitePotentialWell3D.jl).

```@eval
using Markdown
using Antique
Markdown.parse(Antique.load("../../test/result/InfinitePotentialWell3D.log"))
```