```@meta
CurrentModule = Antique
```

# Morse Potential

The Morse potential is a model for inter-nuclear anharmonic vibration in a diatomic molecule.

## Definitions

This model is described with the time-independent Schrödinger equation
```math
  \hat{H} \psi(r) = E \psi(r),
```
and the Hamiltonian
```math
  \hat{H} = - \frac{\hbar^2}{2\mu} \frac{\mathrm{d}^2}{\mathrm{d}r ^2} + V(r)
```
Parameters are specified with the following struct.

#### Parameters
```@docs; canonical=false
Antique.MorsePotential
```

#### Potential
```@docs; canonical=false
Antique.V(::MorsePotential, ::Any)
```

#### Eigen Values
```@docs; canonical=false
Antique.E(::MorsePotential)
```

#### Number of Bound States
```@docs; canonical=false
Antique.nₘₐₓ(::MorsePotential)
```

#### Eigen Functions
```@docs; canonical=false
Antique.ψ(::MorsePotential, ::Any)
```

#### Generalized Laguerre Polynomials
!!! note
    The generalized Laguerre polynomials $L_n^{(\alpha)}(x)$, not the associated Laguerre polynomials $L_n^{k}(x)$, are used in this model. 

```@docs; canonical=false
Antique.L(::MorsePotential, ::Any)
```

#### References
- [P. M. Morse, Phys. Rev. 34, 57 (1929)](https://doi.org/10.1103/PhysRev.34.57)
- [J. P. Dahl, M. Springborg, J. Chem. Phys. 88, 4535 (1988). (62), (63)](https://doi.org/10.1063/1.453761)
- [W. K. Shao, Y. He, J. Pan, J. Nonlinear Sci. Appl., 9, 5, 3388 (2016). (1.6)](http://dx.doi.org/10.22436/jnsa.009.05.124) 
- The Digital Library of Mathematical Functions (DLMF) [18.3 Table1](https://dlmf.nist.gov/18.3#T1), [18.5 Table1](https://dlmf.nist.gov/18.5#T1), [18.5.12](https://dlmf.nist.gov/18.5#E12)

## Usage & Examples

[Install Antique.jl](@ref Install) for the first use and run `using Antique` before each use. The energy `E()`, wavefunction `ψ()`, potential `V()` and some other functions are suppoted. In this system, the model is generated by `MorsePotential` and several parameters `rₑ`, `Dₑ`, `k`, `µ` and `ℏ` are set as optional arguments.

```@example MP
# Parameters for H₂⁺
# https://doi.org/10.1002/slct.202102509
# https://doi.org/10.5281/zenodo.5047817
# https://physics.nist.gov/cgi-bin/cuu/Value?mpsme
rₑ =  1.997193319969992120068298141276
Dₑ = - 0.5 - (-0.602634619106539878727562156289)
k = 2*((-1.1026342144949464615+1/2.00) - (-0.602634619106539878727562156289)) / (2.00 - rₑ)^2
µ = 1/(1/1836.15267343 + 1/1836.15267343)
ℏ = 1.0

using Antique
MP = MorsePotential(rₑ=rₑ, Dₑ=Dₑ, k=k, µ=µ, ℏ=ℏ)
; #hide
```

Parameters:

```@repl MP
MP.rₑ
MP.Dₑ
MP.k
MP.µ
MP.ℏ
```

Number of bound states:

```@repl MP
nₘₐₓ(MP)
```

Eigen values:

```@repl MP
E(MP, n=0)
E(MP, n=1)
```

Potential energy curve:

```@example MP
using CairoMakie

f = Figure()
ax = Axis(f[1,1], xlabel=L"$r$", ylabel=L"$V(r)$",  limits=(0.0,20.0,-0.11,0.1))
lines!(ax, 0.1:0.01:20, r -> V(MP, r))
f
```

Wave functions:

```@example MP
using CairoMakie

# setting
f = Figure()
ax = Axis(f[1,1], xlabel=L"$r$", ylabel=L"$\psi(r)$")

# plot
w0 = lines!(ax, 0..5, x -> ψ(MP, x, n=0))
w1 = lines!(ax, 0..5, x -> ψ(MP, x, n=1))
w2 = lines!(ax, 0..5, x -> ψ(MP, x, n=2))
w3 = lines!(ax, 0..5, x -> ψ(MP, x, n=3))
w4 = lines!(ax, 0..5, x -> ψ(MP, x, n=4))

# legend
axislegend(ax, [w0, w1, w2, w3, w4], [L"n=0", L"n=1", L"n=2", L"n=3", L"n=4"], position=:lb)

f
```

Potential energy curve, Energy levels, Comparison with harmonic oscillator:

```@example MP
using Antique

# https://physics.nist.gov/cgi-bin/cuu/Value?mmusme
m = 206.7682830
μ = 1 / (1/m + 1/m)
MP = MorsePotential(μ=μ)
# @show nₘₐₓ(MP)

using CairoMakie

# settings
f = Figure()
ax = Axis(f[1,1], xlabel=L"$r$", ylabel=L"$V(r),~E_n,~\psi_n(r) \times 5 + E_n$", aspect=1, limits=(0.5,10,-0.105,0.007))
# hidespines!(ax)
# hidedecorations!(ax)

println(nₘₐₓ(MP))

for n in 0:nₘₐₓ(MP)
  # energy
  EMP = E(MP, n=n)
  lines!(ax, 0.1:0.01:15, r -> EMP > V(MP, r) ? EMP : NaN, color=:black, linewidth=2)
  hlines!(ax, E(MP, n=n), color=:black, linewidth=1, linestyle=:dash)
  # wave function
  lines!(ax, 0..10, x -> E(MP,n=n) + 0.0065*ψ(MP,x,n=n), linewidth=2)
end

#potential
lines!(ax, 0..10, x -> V(MP, x), color=:black, linewidth=2)

f
save("assets/fig/MorsePotential.png", f) # hide
; # hide
```

![](assets/fig/MorsePotential.png)

where, the potential of harmonic oscillator is defined as $V(r) \simeq \frac{1}{2} k (r - r_\mathrm{e})^2 + V_0$.

## Testing

Unit testing and Integration testing were done using computer algebra system ([Symbolics.jl](https://symbolics.juliasymbolics.org/stable/)) and numerical integration ([QuadGK.jl](https://juliamath.github.io/QuadGK.jl/stable/)). The test script is [here](https://github.com/ohno/Antique.jl/blob/main/test/MorsePotential.jl).

```@eval
using Markdown
using Antique
Markdown.parse(Antique.load("../../test/result/MorsePotential.log"))
```